import 'dart:collection';

import 'package:meta/meta.dart';
import 'package:miodi/core/consts.dart';
import 'package:miodi/util/map_merge.dart';

import 'module.dart';
import 'providers.dart';
import '../error/errors.dart';

abstract class MiodiComponent {
  final Map<String, Map<Type, Provider>> factoryConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> singletonConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> customConfiguration = HashMap();

  @mustCallSuper
  MiodiComponent() {
    modules.forEach((module) { _saveModule(module); });
  }

  void _saveModule(MiodiModule module) {
    mergeMapTo(singletonConfiguration, module.singletonConfiguration);
    mergeMapTo(factoryConfiguration, module.factoryConfiguration);
    mergeMapTo(customConfiguration, module.customConfiguration);
  }
  
  List<MiodiModule> get modules;

  T get<T>({String name = defaultProviderName}) {
    if (singletonConfiguration[name]?.containsKey(T) ?? false)
      return singletonConfiguration[name][T].provide() as T;
    if (factoryConfiguration[name]?.containsKey(T) ?? false)
      return factoryConfiguration[name][T].provide() as T;
    if (customConfiguration[name]?.containsKey(T) ?? false)
      return customConfiguration[name][T].provide() as T;
    throw NotFoundProviderException(T);
  }
}

class DefaultMiodiComponent extends MiodiComponent {
  final List<MiodiModule> moduleList;
  DefaultMiodiComponent(this.moduleList);

  @override
  List<MiodiModule> get modules => moduleList;

}