import 'dart:collection';

import 'package:miodi/core/consts.dart';
import 'package:miodi/util/map_merge.dart';

import '../error/errors.dart';
import 'component.dart';
import 'providers.dart';

class MiodiConfiguration {
  final Map<String, Map<Type, Provider>> _factoryConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> _singletonConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> _customConfiguration = HashMap();

  MiodiConfiguration(List<MiodiComponent> components) {
    addComponents(components);
  }

  T provide<T>({String name = defaultProviderName}) {
    if (name == null) name = defaultProviderName;

    if (_singletonConfiguration[name] != null && _singletonConfiguration[name].containsKey(T)){
      return _singletonConfiguration[name][T].provide();
    }
    if (_factoryConfiguration[name] != null && _factoryConfiguration[name].containsKey(T)){
      return _factoryConfiguration[name][T].provide();
    }
    if (_customConfiguration[name] != null && _customConfiguration.containsKey(T)){
      return _customConfiguration[name][T].provide();
    }
    throw NotFoundProviderException(T);
  }

  void addComponents(List<MiodiComponent> components) {
    components.forEach((component) {
      mergeMapTo(_singletonConfiguration, component.singletonConfiguration);
      mergeMapTo(_factoryConfiguration, component.factoryConfiguration);
      mergeMapTo(_customConfiguration, component.customConfiguration);
    });
  }
}