import 'dart:collection';

import 'package:meta/meta.dart';
import 'package:miodi/util/map_merge.dart';

import 'providers.dart';

abstract class MiodiModule {
  final Map<String, Map<Type, Provider>> factoryConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> singletonConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> customConfiguration = HashMap();

  @mustCallSuper
  MiodiModule() {
    providers?.forEach((provider) { _saveProvider(provider); });
  }

  void _saveProvider(Provider provider) {
    final name = provider.name;
    Map<String, Map<Type, Provider>> config;

    if (provider is SingletonProvider) {
      config = singletonConfiguration;
    } else if (provider is FactoryProvider) {
      config = factoryConfiguration;
    } else {
      config = customConfiguration;
    }

    if (!config.containsKey(name)) {
      config[name] = {};
    }

    mergeMapTo(config, {name: {provider.provideType: provider}});
  }

  List<Provider> get providers;
}

class DefaultMiodiModule extends MiodiModule{
  final List<Provider> providersList;

  DefaultMiodiModule({this.providersList});

  @override
  List<Provider> get providers => providersList;
}