import 'package:miodi/core/consts.dart';

class FactoryProvider<T> extends Provider<T> {
  FactoryProvider(Type T, Creator<T> creator, String name): super(T, creator, name);

  @override
  T provide() => creator();
}

class SingletonProvider<T> extends Provider<T> {
  T _instance;
  SingletonProvider(Type T, Creator<T> creator, String name): super(T, creator, name);

  void init() {
    _instance = creator();
  }

  @override
  T provide() {
    if (_instance == null) _instance = creator();
    return _instance;
  }
}

abstract class Provider<T> {
  final Type _type;
  final String name;
  final Creator<T> _creator;
  Provider(this._type, this._creator, this.name);
  Type get provideType => _type;
  Creator<T> get creator => _creator;

  T provide();
}

typedef T Creator<T>();