library miodi;

import 'core/consts.dart';

import 'core/component.dart';
import 'core/configuration.dart';
import 'core/module.dart';
import 'core/providers.dart';
import 'error/errors.dart';

export 'core/component.dart';
export 'core/module.dart';
export 'core/providers.dart';

Provider<T> singleton<T>(Type T, Creator<T> creator,
        {String name = defaultProviderName}) =>
    SingletonProvider(T, creator, name);

Provider<T> factory<T>(Type T, Creator<T> creator,
        {String name = defaultProviderName}) =>
    FactoryProvider(T, creator, name);

MiodiModule module({List<Provider> providers}) => DefaultMiodiModule(providersList: providers);

T inject<T>([String name]) => Miodi.inject<T>(name);

class Miodi {
  final MiodiConfiguration _configuration;
  static Miodi _instance;

  Miodi._(this._configuration);

  static void init(List<MiodiComponent> components) {
    if (_instance == null)
      _instance = Miodi._(MiodiConfiguration(components));
    else
      _addComponents(components);
  }

  static void _addComponents(List<MiodiComponent> components) {
    if (_instance == null) throw NotInitializeMiodiException();
    _instance._configuration.addComponents(components);
  }

  static T inject<T>([String name]) {
    if (_instance == null) throw NotInitializeMiodiException();
    return _instance._configuration.provide<T>(name: name);
  }
}
