import 'package:miodi/miodi.dart';
import 'package:test/test.dart';

void main() {
  Miodi.init([appComponent, TestComponent()]);

  test('Test factory providers', () {
    User user1 = Miodi.inject();
    expect(userCounter, equals(1));
    User user2 = Miodi.inject('provider_name');
    expect(userCounter, equals(2));
    User user3 = Miodi.inject();
    expect(userCounter, equals(3));
  });

  test('Test singleton providers', () {
    Database database1 = Miodi.inject();
    expect(databaseCounter, 1);
    Database database2 = Miodi.inject();
    expect(databaseCounter, 1);
    Database database3 = Miodi.inject();
    expect(databaseCounter, 1);
  });

  test('Test dependency', () {
    final block = Miodi.inject<Block>();
    expect(block.repository.database.runtimeType, DatabaseImpl);
  });

  test('Test component sync', () {
    final string = Miodi.inject<String>();
    expect(string, "test");
  });

  return;
}

final appComponent = AppComponent();

class AppComponent extends MiodiComponent {
  @override
  List<MiodiModule> get modules => [
    module(providers: [
      singleton(Database, () => DatabaseImpl()),
      factory(User, () => User()),
      factory(User, () => User(), name: 'provider_name'),
    ]),
    module(
      providers: [
        singleton(Repository, () => Repository(get()))
      ]
    ),
    module(
      providers: [
        factory(Block, () => Block(get()))
      ]
    )
  ];
}

class TestComponent extends MiodiComponent {
  @override
  List<MiodiModule> get modules => [
    module(providers: [
      singleton(String, () => "test")
    ])
  ];
}

class Repository {
  final Database database;

  Repository(this.database);
}

class Block {
  final Repository repository;

  Block(this.repository);
}

var databaseCounter = 0;
abstract class Database {
  void printDatabaseCounter();
}

class DatabaseImpl implements Database {
  DatabaseImpl() {
    databaseCounter++;
    printDatabaseCounter();
  }

  @override
  void printDatabaseCounter() {
    print("Database: " + databaseCounter.toString());
  }
}

var userCounter = 0;
class User {
  User() {
    userCounter++;
    print('User: $userCounter');
  }
}

